unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, OleServer, MSXML2_TLB, ActiveX, ComObj, frmshape;

type
  TfrmSAX = class(TForm)
    btnParseXML: TButton;
    mXML: TMemo;
    cbUseInvalid: TCheckBox;
    procedure btnParseXMLClick(Sender: TObject);
  private
    sax: IVBSAXXMLReader;
  public
  end;

var
  frmSAX: TfrmSAX;

implementation

{$R *.DFM}

var
  SAXEventLog: TStrings;

type
  TMySaxHandler = class (TInterfacedObject, IVBSAXContentHandler)
  public
    // IInterface
    function GetTypeInfoCount(out Count: Integer): HResult; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer;
      NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;

    // IVBSAXContentHandler
    procedure _Set_documentLocator(const Param1: IVBSAXLocator); virtual; safecall;
    procedure startDocument; virtual; safecall;

    procedure endDocument; virtual; safecall;
    procedure startPrefixMapping(var strPrefix: WideString; var strURI: WideString); virtual; safecall;
    procedure endPrefixMapping(var strPrefix: WideString); virtual; safecall;
    procedure startElement(var strNamespaceURI: WideString; var strLocalName: WideString;
                           var strQName: WideString; const oAttributes: IVBSAXAttributes); virtual; safecall;
    procedure endElement(var strNamespaceURI: WideString; var strLocalName: WideString;
                         var strQName: WideString); virtual; safecall;
    procedure characters(var strChars: WideString); virtual; safecall;
    procedure ignorableWhitespace(var strChars: WideString); virtual; safecall;
    procedure processingInstruction(var strTarget: WideString; var strData: WideString); virtual; safecall;
    procedure skippedEntity(var strName: WideString); virtual; safecall;
  end;

  TMySaxErrorHandler = class (TInterfacedObject, IVBSAXErrorHandler)
  public
    // IInterface
    function GetTypeInfoCount(out Count: Integer): HResult; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer;
      NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;

    // IVBSAXErrorHandler
    procedure error(const oLocator: IVBSAXLocator; var strErrorMessage: WideString;
                    nErrorCode: Integer); safecall;
    procedure fatalError(const oLocator: IVBSAXLocator; var strErrorMessage: WideString;
                         nErrorCode: Integer); safecall;
    procedure ignorableWarning(const oLocator: IVBSAXLocator; var strErrorMessage: WideString;
                               nErrorCode: Integer); safecall;
  end;

(* ************************************************************************* *)

function CM_StripControlCodes (str: WideString): WideString;
var c: integer;
begin
  for c := 1 to Length (str) do
    if Ord(Str[c]) >= 32 then Result:=Result+Str[c];
  Result := Trim(Result);
end;

(* ************************************************************************* *)

procedure TfrmSAX.btnParseXMLClick(Sender: TObject);
begin
  mXML.Clear;
  SAXEventLog := mXML.Lines;

  // Create an instance of the MSXML SAX parser
  sax := CoSAXXMLReader.Create;

  // Attach our content handler...
  sax._Set_contentHandler(TMySaxHandler.Create);

  // Attach our error handler...
  sax._Set_errorHandler(TMySaxErrorHandler.Create);

  // Parse an XML document...
  try
    if cbUseInvalid.checked then
      sax.parseURL('bad_request.xml')
    else
      sax.parseURL('request.xml');
  except
    on E: EOleException  do ShowMessage('SAX Parsing failed...');
  end;
end;

(* ************************************************************************* *)

procedure TMySaxHandler.endDocument;
begin
  SAXEventLog.Add ('EVENT : endDocument');
end;

procedure TMySaxHandler.endElement(var strNamespaceURI, strLocalName,
  strQName: WideString);
begin
  SAXEventLog.Add ('EVENT : endElement ('+strLocalName+')');
end;

procedure TMySaxHandler.startElement(var strNamespaceURI: WideString; var strLocalName: WideString;
                       var strQName: WideString; const oAttributes: IVBSAXAttributes);
begin
  SAXEventLog.Add ('EVENT : startElement ('+strLocalName+')');
end;

procedure TMySaxHandler.startDocument;
begin
  SAXEventLog.Add ('EVENT : startDocument');
end;

procedure TMySaxHandler.characters(var strChars: WideString);
var s : WideString;
begin
  s := CM_StripControlCodes(strChars);
  if s <>'' then SAXEventLog.Add ('EVENT : characters : ' + s );
end;

procedure TMySaxHandler.endPrefixMapping(var strPrefix: WideString);
begin
end;

procedure TMySaxHandler.ignorableWhitespace(var strChars: WideString);
begin

end;

procedure TMySaxHandler.processingInstruction(var strTarget,
  strData: WideString);
begin

end;

procedure TMySaxHandler._Set_documentLocator(const Param1: IVBSAXLocator);
begin

end;

procedure TMySaxHandler.skippedEntity(var strName: WideString);
begin

end;

procedure TMySaxHandler.startPrefixMapping(var strPrefix,
  strURI: WideString);
begin

end;

(* ************************************************************************* *)

function TMySaxHandler.Invoke(DispID: Integer; const IID: TGUID;
  LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo,
  ArgErr: Pointer): HResult;
begin
  Result := E_NOTIMPL;
end;

function TMySaxHandler.GetIDsOfNames(const IID: TGUID; Names: Pointer;
  NameCount, LocaleID: Integer; DispIDs: Pointer): HResult;
begin
  Result := E_NOTIMPL;
end;

function TMySaxHandler.GetTypeInfo(Index, LocaleID: Integer;
  out TypeInfo): HResult;
begin
  Result := E_NOTIMPL;
end;

function TMySaxHandler.GetTypeInfoCount(out Count: Integer): HResult;
begin
  Result := E_NOTIMPL;
end;

(* ************************************************************************* *)

procedure TMySaxErrorHandler.error(const oLocator: IVBSAXLocator;
  var strErrorMessage: WideString; nErrorCode: Integer);
begin
  SAXEventLog.Add ('[Error] ' +  IntToStr (oLocator.lineNumber) + '.' +
                                 IntToStr (oLocator.columnNumber) + ':' +
                                 strErrorMessage);
end;

procedure TMySaxErrorHandler.fatalError(const oLocator: IVBSAXLocator;
  var strErrorMessage: WideString; nErrorCode: Integer);
begin
  SAXEventLog.Add ('[Fatal] ' +  IntToStr (oLocator.lineNumber) + '.' +
                                 IntToStr (oLocator.columnNumber) + ':' +
                                 strErrorMessage);
end;

procedure TMySaxErrorHandler.ignorableWarning(
  const oLocator: IVBSAXLocator; var strErrorMessage: WideString;
  nErrorCode: Integer);
begin
  SAXEventLog.Add ('[Warning] ' + IntToStr (oLocator.lineNumber) + '.' +
                                  IntToStr (oLocator.columnNumber) + ':' +
                                  strErrorMessage);
end;

function TMySaxErrorHandler.GetIDsOfNames(const IID: TGUID; Names: Pointer;
  NameCount, LocaleID: Integer; DispIDs: Pointer): HResult;
begin
  Result := E_NOTIMPL;
end;

function TMySaxErrorHandler.GetTypeInfo(Index, LocaleID: Integer;
  out TypeInfo): HResult;
begin
  Result := E_NOTIMPL;
end;

function TMySaxErrorHandler.GetTypeInfoCount(out Count: Integer): HResult;
begin
  Result := E_NOTIMPL;
end;

function TMySaxErrorHandler.Invoke(DispID: Integer; const IID: TGUID;
  LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo,
  ArgErr: Pointer): HResult;
begin
  Result := E_NOTIMPL;
end;

(* ************************************************************************* *)

end.
